'''
Classes to handle Services on NetOrca
'''

import netorca_sdk.constants as const
from netorca_sdk.base import BaseObj, Endpoint
from netorca_sdk.elements import teams


class ServiceObj(BaseObj):
    def __init__(self, *args, **kwargs):
        self.class_map = {
            "owner": {'module': teams, 'name': 'TeamObj'}
        }
        super().__init__(*args, **kwargs)

class Services(Endpoint):
    def __init__(self, conn, context='owner'):
        '''
        Initialises the class
        :param conn: Connection object
        :param context: str -> consumer or serviceowner context
        '''
        self.settings = {
            'owner_path': const._PATH_SERVICE_OWNER,
            'consumer_path': const._PATH_SERVICE_CONSUMER,
            'filters': [
                'owner_uuid'
            ]
        }
        self.class_ = ServiceObj
        super().__init__(conn, context)

    def get_service_by_name(self, name):
        '''
        Returns a ServiceObj that matches the inputted name.
        :param name: str -> name of the Service
        :return: ServiceObj
        '''
        self.get_all()
        for obj in self.objs:
            if obj.name == name:
                return obj
        return None

# class Services:
#     '''
#     Class to connect to the ChangeInstances API
#     '''
#     def __init__(self, conn, context='owner'):
#         '''
#         Initialise
#         :param conn: base.Connection instance
#         :param serivce: uuid of the Service to filter by
#         '''
#         self.conn = conn
#         self.context = context
#         if self.context=='owner':
#             self.path = const._PATH_SERVICE_OWNER
#         elif self.context=='consumer':
#             self.path = const._PATH_SERVICE_CONSUMER
#         else:
#             raise AttributeError('context type not supported')
#         self.objs = []
#
#     def get_all(self):
#         '''
#         Gets all Services
#         :return: None
#         '''
#         response = self.conn.get(path=self.path)
#         if response.status_code == 200 and response.json()['results']:
#             for item in response.json()['results']:
#                 self.objs.append(ServiceObj(self.conn, self.context, item))
#         else:
#             raise ConnectionError('No results return from ServiceAPI, something went wrong')
#
#     def get_service_by_name(self, name):
#         '''
#         Returns a ServiceObj that matches the inputted name.
#         :param name: str -> name of the Service
#         :return: ServiceObj
#         '''
#         self.get_all()
#         for obj in self.objs:
#             if obj.name == name:
#                 return obj
#         return None
#
#     def get_single(self, uuid):
#         '''
#         Returns a single change_instance based on uuid or id
#         :param uuid: str -> uuid
#         :return: ChangeInstanceObj instance
#         '''
#         url = urljoin(self.path, uuid)
#         response = self.conn.get(path=url)
#         if response.status_code==200:
#             return ServiceObj(self.conn, self.context, response.json())
#         else:
#             logger.warning('Service not found')
#             return None


'''
Contains classes for interactions with teams on NetOrca
'''
from netorca_sdk.base import BaseObj

class TeamsObj(BaseObj):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)




'''
Classes to handle ChangeInstances on NetOrca
'''

import logging
import netorca_sdk.constants as const
from netorca_sdk.base import BaseObj, Endpoint
from netorca_sdk.elements import teams, applications
from netorca_sdk.elements import declarations, sis
from urllib.parse import urljoin
from netorca_sdk.utils.decorators import owner_required

logger = logging.getLogger(__name__)

class ChangeInstanceObj(BaseObj):
    '''
    Change instance object
    '''
    def __init__(self, *args, **kwargs):
        self.class_map = {
            "service_item": {'module': sis, 'name': 'ServiceItemObj'},
            "new_declaration": {'module': declarations, 'name': 'DeclarationObj'},
            "owner": {'module': teams, 'name': 'TeamsObj'},
            "application": {'module': applications, 'name': 'ApplicationObj'}
        }
        super().__init__(*args, **kwargs)
        self._validate_input()
        self._create_paths()

    def _validate_input(self):
        '''
        Makes sure that required field exist in the change instance input
        :return: None
        '''
        required_fields = ['uuid', 'state']
        if not all(i in self.input.keys() for i in required_fields):
            print('hit')
            raise ValueError('Required fields were not provided in ChangeInstanceObj initialisation')

    def _create_paths(self):
        '''
        Creates the paths for any updates
        :return: None
        '''
        if self.context == 'owner':
            self.base_path = const._PATH_CHANGE_INSTANCES_OWNER
        elif self.context == 'consumer':
            self.base_path = const._PATH_CHANGE_INSTANCES_CONSUMER
        else:
            raise ValueError(f'Context f{ self.context } is not supported')

    def _modify_change_state(self, new_state, deployed_item):
        '''
        Put request to modify the change state
        :return:
        '''
        if deployed_item:
            data = {
                'state': new_state,
                'deployed_item': deployed_item
            }
        else:
            data = {
                'state': new_state,
                'deployed_item': {}
            }
        path = urljoin(self.base_path, self.uuid+'/')
        response = self.conn.put(path, data)
        if response.status_code == 200:
            return new_state
        else:
            logger.warning(f'Failed to update Change Instance {self.uuid}')
            return None
        # Update to completed here

    @owner_required
    def complete_change(self, deployed_item={}):
        '''
        Mark change as completed
        :param deployed_item: dict -> deployed item dictionary
        :return: new state
        '''
        if self.state == const.NETORCA_STATES_APPROVED:
            return self._modify_change_state(const.NETORCA_STATES_COMPLETED, deployed_item)
        else:
            logger.warning('Cannot complete change, not in correct state')
            return None

    @owner_required
    def error_change(self, deployed_item={}):
        '''
        Mark change as errored
        :param deployed_item:
        :return:
        '''
        if self.state != const.NETORCA_STATES_COMPLETED or self.state != const.NETORCA_STATES_ERROR:
            return self._modify_change_state(const.NETORCA_STATES_ERROR, deployed_item)
        else:
            logger.warning('Cannot complete change, not in correct state')
            return None

            # Update change state here

    @owner_required
    def approve_change(self, deployed_item={}):
        '''
        Mark change as approved
        :param deployed_item:
        :return:
        '''
        if self.state == const.NETORCA_STATES_PENDING:
            # Update to approved here
            return self._modify_change_state(const.NETORCA_STATES_APPROVED, deployed_item)
        else:
            logger.warning('Cannot approve change, it not in the PENDING state')
            return None


class ChangeInstances(Endpoint):
    def __init__(self, conn, context='owner'):
        '''
        Initialises the class
        :param conn: Connection object
        :param context: str -> consumer or serviceowner context
        '''
        self.settings = {
            'owner_path': const._PATH_CHANGE_INSTANCES_OWNER,
            'consumer_path': const._PATH_CHANGE_INSTANCES_CONSUMER,
            'filters': [
                'consumer_owner_uuid',
                'service_owner_uuid',
                'application_uuid',
                'service_uuid',
                'service_item_uuid',
                'request_uuid'
            ]
        }
        self.class_ = ChangeInstanceObj
        super().__init__(conn, context)

# class ChangeInstances:
#     '''
#     Class to connect to the ChangeInstances API
#     '''
#     def __init__(self, conn, context='owner'):
#         '''
#         Initialise
#         :param conn: base.Connection instance
#         '''
#         self.conn = conn
#         self.context = context
#         if self.context=='owner':
#             self.path = const._PATH_CHANGE_INSTANCES_OWNER
#         elif self.context=='consumer':
#             self.path = const._PATH_CHANGE_INSTANCES_CONSUMER
#         else:
#             raise AttributeError('context type not supported')
#         self.objs = []
#
#     def get_all(self, state=None, service_uuid=None):
#         '''
#         Gets all the change instances
#         :param state: str -> state to filter by
#         :param service_uuid: str -> uuid of a Service to filter by
#         :return:
#         '''
#         filters = {}
#         if state:
#             filters['state'] = state
#         if service_uuid:
#             filters['service_uuid'] = service_uuid
#         response = self.conn.get(path=self.path, filters=filters)
#         if response.status_code == 200 and response.json()['results']:
#             for item in response.json()['results']:
#                 self.objs.append(ChangeInstanceObj(self.conn, self.context, item))
#         else:
#             raise ConnectionError('No results return from ChangeInstance API, something went wrong')
#
#     def get_single(self, uuid):
#         '''
#         Returns a single change_instance based on uuid or id
#         :param uuid: str -> uuid
#         :return: ChangeInstanceObj instance
#         '''
#         url = urljoin(self.path, uuid)
#         response = self.conn.get(path=url)
#         if response.status_code==200:
#             return ChangeInstanceObj(self.conn, self.context, response.json())
#         else:
#             logger.warning('CI not found')
#             return None










#!/usr/bin/env python3
'''
This module contains the methods to connect to NetOrca platform
and carry out basic functions usual for ansible integrations.
'''

import logging
from urllib.parse import urljoin
import requests

import netorca_sdk.constants as const


class Connection:
    '''
    Base class to handle connectivity and basic functions to NetOrca
    '''
    def __init__(self, base_url, username=None,  password=None, api_key=None):
        '''
        Initialises the class and connection to the NetOrca platform
        :param base_url: url of NetOrca deployment
        :param username: Username for NetOrca login
        :param password: Password for NetOrca login
        :param api_key: API_key for NetOrca login (will be used as first preference)
        '''
        self.base_url = base_url
        self.api_key = api_key
        self.username = username
        self.password = password
        self._login()

    def _login(self):
        '''
        Logs into the NetOrca platform via api key or via username and password generating a token
        :return: None
        '''
        url = urljoin(self.base_url, const._PATH_LOGIN)
        if self.api_key:
            self.header = {
                'Authorization': f'Api-Key { self.api_key }',
                'content_type': 'application/json'
            }
            response = requests.get(urljoin(self.base_url, const._PATH_TEAMS), headers=self.header)
            if response.status_code != 200:
                raise ConnectionRefusedError('api_key not valid on the NetOrca deployment')
        else:
            data = {
                "username": self.username,
                "password": self.password
            }
            response = requests.post(url, json=data)
            if response.status_code == 200 and 'token' in response.json():
                token = response.json()['token']
                self.header = {
                    'Authorization': f'Token { response.json()["token"] }',
                    'content-type': 'application/json'
                }
            else:
                raise ConnectionRefusedError('Supplied username and password not accepted by NetOrca platform')

    def post(self, path, data):
        '''
        Post to the NetOrca api
        :param path: NetOrca path
        :param data: Data to post in JSON
        :return: Requests response
        '''
        url = urljoin(self.base_url, path)
        return requests.post(url, json=data, headers=self.header)

    def put(self, path, data):
        '''
        Put to the NetOrca API
        :param path: NetOrca path
        :param data: Data to put in JSON
        :return: Request reponse
        '''
        url = urljoin(self.base_url, path)
        return requests.put(url, json=data, headers=self.header)

    def get(self, path, filters=None):
        '''
        Gets from the netorca api
        :param path: NetOrca path
        :param filters: dict of filters to apply
        :return: Requests reponse
        '''
        url = urljoin(self.base_url, path)
        return requests.get(url, params=filters, headers=self.header)


class BaseObj:
    '''
    Base NetOrca Object
    '''
    def __init__(self, conn, context, input):
        '''
        Initialise the object
        '''
        self.conn = conn
        self.input = input
        self.context=context
        self._map_fields()

    def _map_fields(self):
        '''
        Map fields from the input to methods
        :return: None
        '''
        try:
            class_map = getattr(self, 'class_map')
            for key, value in self.input.items():
                if key in class_map.keys():
                    class_ = getattr(class_map[key]['module'], class_map[key]['name'])
                    setattr(self, key, class_(self.conn, self.context, value))
                else:
                    setattr(self, key, value)
        except AttributeError:
            for key, value in self.input.items():
                setattr(self, key, value)


class Endpoint:
    '''
    Base class to handle endpoint connection to the NetOrca api
    '''
    def __init__(self, conn, context):
        '''
        Endpoint base class, requires an initialised NetOrca connection class
        :param connection: Connection cls
        '''
        self.conn = conn
        self.context = context
        try:
            getattr(self, 'settings')
            getattr(self, 'class_')
        except AttributeError:
            raise RuntimeError('Endpoint class should not be used directly')
        self._set_path()
        self.objs = []

    def _set_path(self):
        '''
        Sets the path based on the context input and settings
        :return: None
        '''
        try:
            if self.context == 'owner':
                self.path = self.settings['owner_path']
            elif self.context == 'consumer':
                self.path = self.settings['consumer_path']
            else:
                raise RuntimeError('Context not supported')
        except KeyError:
            raise RuntimeError('Incorrectly configured class')

    def supported_filters(self):
        '''
        Returns a list of supported filters on this cllass
        :return: list
        '''
        try:
            return self.settings['filters']
        except KeyError:
            logging.warning('No filters specified for this class')
            return None

    def get_all(self, **kwargs):
        '''
        Gets all items for this endpoint type
        :param kwargs: named filter arguments
        :return: None
        '''
        filters = kwargs
        response = self.conn.get(path=self.path, filters=filters)
        if response.status_code == 200:
            for item in response.json()['results']:
                self.objs.append(self.class_(self.conn, self.context, item))
        else:
            raise ConnectionError(f' { response.status_code } return from API, '
                                  f'something went wrong. Log details: \n\n{ response.text }')

    def get_single(self, uuid):
        '''
        Gets a single instance of the endpoint object
        :param uuid: str -> uuid of object
        :return: Obj class instance
        '''
        url = urljoin(self.path, uuid)
        response = self.conn.get(path=url)
        if response.status_code==200:
            return self.class_(self.conn, self.context, response.json())
        else:
            logger.warning('Obj not found')
            return None



'''
Define decorators to be used with any element classes
'''
import logging
import functools

def owner_required(fn):
    @functools.wraps(fn)
    def wrapper(self, *args, **kwargs):
        if self.context == 'owner':
            return fn(self, *args, **kwargs)
        else:
            logging.warning('This function is only permitted in the owner context')
            raise ValueError('This function is only permitted in the owner context')
    return wrapper


'''
Classes to handle ServiceItems on NetOrca
'''

import netorca_sdk.constants as const
from netorca_sdk.base import BaseObj, Endpoint
from netorca_sdk.elements import teams, applications, services
from netorca_sdk.elements import declarations


class ServiceItemObj(BaseObj):
    def __init__(self, *args, **kwargs):
        self.class_map = {
            "service": {'module': services, 'name': 'ServiceObj'},
            "declaration": {'module': declarations, 'name': 'DeclarationObj'},
            "team": {'module': teams, 'name': 'TeamsObj'},
            "application": {'module': applications, 'name': 'ApplicationObj'}
        }
        super().__init__(*args, **kwargs)

    def __repr__(self):
        return f"ServiceItemObj(name={self.name},application={self.application},uuid={self.uuid},id={self.id})"

    def _create_paths(self):
        '''
        Creates the paths for any updates
        :return: None
        '''
        if self.context == 'owner':
            self.base_path = const._PATH_SERVICE_ITEMS_OWNER
        elif self.context == 'consumer':
            self.base_path = const._PATH_SERVICE_ITEMS_CONSUMER
        else:
            raise ValueError(f'Context f{ self.context } is not supported')


class ServiceItems(Endpoint):
    def __init__(self, conn, context='owner'):
        '''
        Initialises the class
        :param conn: Connection object
        :param context: str -> consumer or serviceowner context
        '''
        self.settings = {
            'owner_path': const._PATH_SERVICE_ITEMS_OWNER,
            'consumer_path': const._PATH_SERVICE_ITEMS_CONSUMER,
            'filters': [
                'consumer_owner_uuid',
                'service_owner_uuid',
                'service_uuid',
                'service_name',
                'application_uuid',
                'runtime_state',
                'change_state'
            ]
        }
        self.class_ = ServiceItemObj
        super().__init__(conn, context)

# class ServiceItems:
#     '''
#     Class to connect to the ServiceItems API
#     '''
#     def __init__(self, conn, context='owner'):
#         '''
#         Initialise
#         :param conn: base.Connection instance
#         :param serivce: str -> name of service (if NONE gets all available services)
#         '''
#         self.conn = conn
#         self.context = context
#         if self.context=='owner':
#             self.path = const._PATH_SERVICE_ITEMS_OWNER
#         elif self.context=='consumer':
#             self.path = const._PATH_SERVICE_ITEMS_CONSUMER
#         else:
#             raise AttributeError('context type not supported')
#         self.objs = []
#
#     def get_all(self, service_uuid):
#         '''
#         Gets all change instances for a particular state
#         :return: None
#         '''
#         filters = {}
#         if service_uuid:
#             filters['service_uuid'] == service_uuid
#         response = self.conn.get(path=self.path, fitlers=filters)
#         if response.status_code == 200 and response.json()['results']:
#             for item in response.json()['results']:
#                 self.objs.append(ServiceItemObj(self.conn, self.context, item))
#         else:
#             raise ConnectionError('No results return from ChangeInstance API, something went wrong')
#
#
#     def get_single(self, uuid):
#         '''
#         Returns a single service item by uuid
#         :param uuid:
#         :return: ServiceItemObj
#         '''
#         url = urljoin(self.path, uuid)
#         response = self.conn.get(path=url)
#         if response.status_code == 200:
#             return ServiceItemObj(self.conn, self.context, response.json())
#         else:
#             logger.warning('SI not found')
#             return None


from distutils.core import setup
setup(
  name = 'netorca_sdk',
  packages = ['netorca_sdk', 'netorca_sdk.elements', 'netorca_sdk.utils'],
  version = '0.0.6',
  license='MIT',
  description = 'A package for interacting with the NetOrca API',
  author = 'Scott Rowlandson',
  author_email = 'scott@netautomate.org',
  url = 'https://gitlab.com/netorca_public/netorca_sdk/',
  keywords = ['netorca', 'orchestration', 'netautomate'],
  install_requires=[
          'requests',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11',
  ],
)


''''
Contains the Constants used by ansible modules
'''
NETORCA_STATES_ERROR = 'ERROR'
NETORCA_STATES_PENDING = 'PENDING'
NETORCA_STATES_APPROVED = 'APPROVED'
NETORCA_STATES_COMPLETED = 'COMPLETED'
NETORCA_VALID_STATES = [
    NETORCA_STATES_ERROR,
    NETORCA_STATES_PENDING,
    NETORCA_STATES_APPROVED,
    NETORCA_STATES_COMPLETED
]

FIELDS_URL = 'url'
FIELDS_USER = 'username'
FIELDS_API_KEY = 'token'
FIELDS_PASS = 'password'
FIELDS_STATE = 'state'
FIELDS_SERVICE = 'service_name'
FIELDS_UUID = 'uuid'
FIELDS_DEPLOYED_ITEM = 'deployed_item'


# WARNING: PATHs must not be prefixed with slash (/) otherwise urljoin won't work correctly
_PATH_LOGIN = "api-token-auth/"
_PATH_CHANGE_INSTANCES_OWNER = "orcabase/serviceowner/change_instances/"
_PATH_CHANGE_INSTANCES_CONSUMER = "orcabase/consumer/change_instances/"
_PATH_SERVICE_ITEMS_OWNER = "orcabase/serviceowner/service_items/"
_PATH_SERVICE_ITEMS_CONSUMER = "orcabase/consumer/service_items/"
_PATH_SERVICE_OWNER = "orcabase/serviceowner/services/"
_PATH_SERVICE_CONSUMER = "orcabase/consumer/services/"
_PATH_TEAMS = "account/teams/"

